% Load cell-test data. Contains variable "DYNData" of which the field
% "script1" is of interest. It has sub-fields time, current, voltage, soc.
load dynModel.mat
model.R0Param = 0.032;
model.RParam = -model.RParam;
model.QParam = model.QParam*1000;

% load any dynamic data in the 'data' folder
load('data/25degC_UDDS.mat');
T = 25; % test temperature = 25 degC


time = meas.Time;
deltat = 0.1;
time = time-time(1);
current =  -meas.Current;
voltage = meas.Voltage;
power = voltage.*current;
sigma = 0.1369;
mu_0 = 0.0198;
M = 5;
h = 9.2;

% voltage(15000:15200) = voltage(15000:15200)*1.1;  %%Uncomment this line
% if you want to test the fault detection algorithm
soc = (model.QParam + meas.Ah)./model.QParam;

sochat = zeros(size(soc)); 
socbound = zeros(size(soc));
yhat = zeros(size(soc));
ir = zeros(size(soc));
g = zeros(size(soc));
error = zeros(size(soc));
alarm = zeros(size(soc));

% Covariance values
SigmaX0 = diag([1e-2 6e-3 1e-5]); % uncertainty of initial state
SigmaW = 2; % uncertainty of current sensor, state equation
SigmaV = 100; % uncertainty of voltage sensor, output equation


ekfData   = initEKF(voltage(1),T,SigmaX0,SigmaV,SigmaW,model);

% Now, enter loop for remainder of time, where we update the EKF
% once per sample interval
hwait = waitbar(0,'Computing...');

for k = 1:length(voltage)
vk = voltage(k); % "measure" voltage
ik = current(k); % "measure" current
Tk = T; % "measure" temperature

% Update SOC (and other model states)
[sochat(k),socbound(k),yhat(k),ir(k),ekfData,errorflag] = iterEKF(vk,ik,Tk,deltat,ekfData);
error(k) = yhat(k) - vk ; 
if k >5
    g(k) = 1/(2*sigma^2*M) * (sum(error(k-5:k) - mu_0))^2;
end
% if g(k) > h
%     fprintf('Voltage sensor fault detected at %g min %%\n',time(k)/60);
%     alarm(k) = 1;
% end
if mod(k,1000)==0, waitbar(k/length(current),hwait); end
end

close(hwait);
close all
figure(1); clf; plot(time,100*sochat,time,100*soc,'LineWidth',2); hold on
h = plot([time; NaN; time],...
[100.*(sochat+socbound); NaN; 100.*(sochat-socbound)],'LineWidth',2);
title('SOC estimation using EKF');
xlabel('Time (s)'); ylabel('SOC (%)');
ylim([0 105]);
legend('Estimate','Truth','Bounds'); grid on
fprintf('RMS SOC estimation error = %g%%\n',...
sqrt(mean((100*(soc-sochat)).^2)));
fprintf('Average SOC estimation error = %g %%\n',...
    abs(mean((100*(soc-sochat)).^2)));
set(gca,'FontSize',14)



figure(2); clf; plot(time,100*(soc-sochat),'LineWidth',2); hold on
h = plot([time; NaN; time],[100*socbound; NaN; -100*socbound],'LineWidth',2);
title('SOC estimation errors using EKF');
xlabel('Time (s)'); ylabel('SOC error (%)'); ylim([-4 4]);
legend('Estimation error','Bounds'); grid on
ind = find(abs(soc-sochat)>socbound);
fprintf('Percent of time error outside bounds = %g%%\n',...
    length(ind)/length(soc)*100);
set(gca,'FontSize',14)


error_voltage = yhat - voltage;
error_voltage = error_voltage(1:120000);
fprintf('measurement error: mean = %g, variance = %g \n', ...
    mean(error_voltage), std(error_voltage));
fprintf('mu_1 = %g\n', mean(error_voltage(15000:15200)));
figure(3); histfit(error_voltage);
xlabel('Error (V)');
title('Error histogram');
set(gca,'FontSize',14)
xlim([-0.1 0.2]);

figure(4); clf; 
plot(time(1:120000)/60,error_voltage,'LineWidth',2);
xlabel('Time (min)'); ylabel('Voltage error (V)');
set(gca,'FontSize',14)
ylim([-.6 .4]); 

figure(5); 
plot(time(1:120000)/60,yhat(1:120000),'LineWidth',2); hold on
plot(time(1:120000)/60,voltage(1:120000),'LineWidth',2);
legend('Voltage estimation','Voltage measurement');
set(gca,'FontSize',14)
xlabel('Time (min)'); ylabel('Voltage (V)');
ylim([3.2 4.6]);

figure(6);
plot(time(1:120000)/60,g(1:120000),'LineWidth',2);
xlabel('Time (min)'); ylabel('g(k)');
h = yline(9.2,'--r','LineWidth',4);
set(gca,'FontSize',14)


