function [zk,zkbnd,yhat,ir,ekfData,errorflag] = iterEKF(vk,ik,Tk,deltat,ekfData)
model = ekfData.model;
% Load the cell model parameters for the present operating temp.
Q = getParamESC('QParam',Tk,model);
G = getParamESC('GParam',Tk,model);
M = getParamESC('MParam',Tk,model);
M0 = getParamESC('M0Param',Tk,model);
RC = exp(-deltat./abs(getParamESC('RCParam',Tk,model)))';
R = getParamESC('RParam',Tk,model)';
R0 = getParamESC('R0Param',Tk,model);
eta = getParamESC('etaParam',Tk,model);
if ik<0, ik=ik*eta; end; % adjust current if charging cell

% Get data stored in ekfData structure
SigmaX = ekfData.SigmaX; SigmaW = ekfData.SigmaW;
SigmaV = ekfData.SigmaV;
irInd = ekfData.irInd; hkInd = ekfData.hkInd;
zkInd = ekfData.zkInd;
xhat = ekfData.xhat; nx = length(xhat);
I = ekfData.priorI;
if abs(ik)>Q/100, ekfData.signIk = sign(ik); end;
signIk = ekfData.signIk;

errorflag = 0;


% Step 1a: State-prediction time update
% First, compute Ahat[k-1], Bhat[k-1]
Ah = exp(-abs(I*G*deltat/(3600*Q))); % hysteresis factor
Bh = -abs(G*deltat/(3600*Q))*Ah*(1+sign(I)*xhat(hkInd));
Ahat = zeros(nx,nx); 
Bhat = zeros(nx,1);

Ahat(zkInd,zkInd) = 1; 
Bhat(zkInd) = -deltat/(3600*Q);

Ahat(irInd,irInd) = RC; 
Bhat(irInd) = 1-RC;

B = [Bhat, 0*Bhat];
Ahat(hkInd,hkInd) = Ah; 
Bhat(hkInd) = Bh;
B(hkInd,2) = Ah-1;
% Next, update xhat
xhat = Ahat*xhat + B*[I; sign(I)];

% Step 1b: Error-covariance time update
% sigmaminus(k) = Ahat(k-1)*sigmaplus(k-1)*Ahat(k-1)' + ...
% Bhat(k-1)*sigmawtilde*Bhat(k-1)'
SigmaX = Ahat*SigmaX*Ahat' + Bhat*SigmaW*Bhat';


% Step 1c: Output estimate
yhat = OCVfromSOCtemp(xhat(zkInd),model) + M0*signIk + ...
M*xhat(hkInd) - R*xhat(irInd) - R0*ik;


% Step 2a: Estimator gain matrix
Chat = zeros(1,nx);
Chat(zkInd) = OCVfromSOCtemp(xhat(zkInd),model);
Chat(hkInd) = M;
Chat(irInd) = -R;
Dhat = 1;
SigmaY = Chat*SigmaX*Chat' + Dhat*SigmaV*Dhat';
L = SigmaX*Chat'/SigmaY;


% Step 2b: State-estimate measurement update
r = vk - yhat; % residual. Use to check for sensor errors...
if r^2 > 100*SigmaY, L(:)=0.0; end
xhat = xhat + L*r;
% xhat(hkInd) = min(1,max(-1,xhat(hkInd))); % Help maintain robustness
% xhat(zkInd) = min(1.05,max(-0.05,xhat(zkInd)));


% Step 2c: Error-covariance measurement update
SigmaX = SigmaX - L*SigmaY*L';

% SigmaX-bump code
%if r^2 > 2*SigmaY % bad voltage estimate by 2 std devs, bump SigmaX
if r^2 > 0.5
    errorflag = 1;
%SigmaX(zkInd,zkInd) = SigmaX(zkInd,zkInd)*ekfData.SXbump;
end
[~,S,V] = svd(SigmaX);
HH = V*S*V';
SigmaX = (SigmaX + SigmaX' + HH + HH')/4; % Help maintain robustness
% Save data in ekfData structure for next time...
ekfData.priorI = ik;
ekfData.SigmaX = SigmaX;
ekfData.xhat = xhat;
zk = xhat(zkInd);
ir = xhat(irInd);
zkbnd = 3*sqrt(SigmaX(zkInd,zkInd));
end